/*
**  $Id: testgl4.c,v 1.1 1997/09/30 12:00:00 jochen Rel $ 
**  Copyright (C) 1997  Jochen Metzinger<1metzing@informatik.uni-hamburg.de>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software Foundation,
**  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <vga.h>
#include "../src/vgagl4.h"		/* for local testing */


unsigned char TestPattern[9][9] = {
	{15, 4, 4, 4, 4, 4, 4, 4, 4},
	{ 4, 1, 1, 1, 1, 1, 1, 1, 4},
	{ 4, 1, 2, 2, 2, 2, 2, 1, 4},
	{ 4, 1, 2, 0, 0, 0, 2, 1, 4},
	{ 4, 1, 2, 0, 0, 0, 2, 1, 4},
	{ 4, 1, 2, 0, 0, 0, 2, 1, 4},
	{ 4, 1, 2, 2, 2, 2, 2, 1, 4},
	{ 4, 1, 1, 1, 1, 1, 1, 1, 4},
	{ 4, 4, 4, 4, 4, 4, 4, 4,15}};

#define	VGAMODE	G640x350x16

int
main()
{
	int y0 = 0;
	int x, y, i;
	char *bp;

	if (vga_init() != 0) {
		exit(1);
	}
	if (vga_hasmode(VGAMODE) != 1) {
		fprintf(stderr, "vga mode %s not available",
			vga_getmodename(VGAMODE));
		exit(2);
	}
	vga_setmode(VGAMODE);
	gl4_setcontextvga(VGAMODE);
	bp = malloc(WIDTH*HEIGHT);
	/* setpixel */
	for(y=0; y<16; y++) {
		for (x=0; x<WIDTH; x++) {
			gl_setpixel(x, y+y0, x+y);
		}
	}
	y0 += 16;
	/* getpixel */
	for(y=0; y<16; y++) {
		for (x=0; x<WIDTH; x++) {
			gl_setpixel(x, y+y0, ~gl_getpixel(x,y+y0-16));
		}
	}
	y0 += 16;
	/* hline */
	for(i=0; i<16; i++) {
		gl_hline(   i, i+y0, i+32, (i!=0)?i:15);
		gl_hline(64-i, i+y0, 128-i, (i==15)?15:15-i);
	}
	y0 += 16;
	/* fillbox */
	for(i=0; i<16; i++) {
		gl_fillbox(   i, i+y0, 32, 1, i);
		gl_fillbox(64-i, i+y0, 32, 1, 15-i);
	}
	y0 += 16;
	gl_fillbox(0, y0, WIDTH, HEIGHT-y0, 9);
	/* putbox */
	for (x=0; x<=WIDTH-9; x+=9) {
		gl_putbox(x, y0, 9, 9, TestPattern);
	}
	y0 += 16;
	/* getbox */
	gl_getbox(0, -16+y0, WIDTH, 16, bp);
	gl_putbox(0, y0, WIDTH, 16, bp);
	y0 += 16;
	/* putboxmask */
	for (x=0; x<=WIDTH-9; x+=9) {
		gl_putboxmask(x, y0, 9, 9, TestPattern);
	}
	y0 += 16;
	/* big get/put-box */
	gl_getbox(0, 0, WIDTH, 64, bp);
	gl_getbox(0, 0, WIDTH, HEIGHT, bp);
	gl_clearscreen(0);
	gl_putbox(0, 0, WIDTH, HEIGHT, bp);
	/* putboxpart */
	gl_putboxpart(5, HEIGHT-16, WIDTH-50, 16, WIDTH, HEIGHT, bp, 16, 16);
	/* getboxpart */
	memset(bp, 0, WIDTH*32);
	gl_setclippingwindow(16, 16, WIDTH-1, HEIGHT-1);
	gl_getbox( 0,  0, WIDTH, 32, bp);
	gl_disableclipping();
	gl_putbox( 0, y0, WIDTH, 32, bp);
	y0 += 32;
	/* copybox */
	gl_fillbox( 9,  9+y0, 22, 22, 15);	/* not copied */
	gl_fillbox(10, 10+y0, 20, 20,  0);
	for (x=1; x<20-1; x++) {
		for (y=1; y<20-1; y++) {
			gl_setpixel(10+x, 10+y+y0, x+y);
		}
	}
	gl_getbox( 9,  9+y0, 22, 22, bp);
	for (y=0; y<3; y++) {
		for (x=0; x<13; x++) {
			gl_putbox(40*x+9, 40*y+9+y0, 22, 22, bp);
		}
	}
	for (y=0; y<3; y++) {
		for (x=0; x<13; x++) {
			gl_copybox(40*x+10, 40*y+10+y0, 20, 20,
				40*x+10+2*(x-4), 40*y+10+y0+8*(y-1));
		}
	}
	/* big copybox */
	gl_copybox(0, 0, WIDTH, HEIGHT-4, 0, 4);
	gl_fillbox(0, 0, WIDTH, 4, 0);
	gl_copybox(0, 4, WIDTH, HEIGHT-4, 0, 0);
	gl_fillbox(0, HEIGHT-4, WIDTH, 4, 0);
	gl_copybox(0, 0, WIDTH-4, HEIGHT, 4, 0);
	gl_fillbox(0, 0, 4, HEIGHT, 0);
	gl_copybox(4, 0, WIDTH-4, HEIGHT, 0, 0);
	gl_fillbox(WIDTH-4, 0, 4, HEIGHT, 0);
	vga_getch();
	vga_setmode(TEXT);
	return 0;
}
